"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const knex_1 = __importDefault(require("knex"));
const yargs = __importStar(require("yargs"));
const analyser_1 = require("./analyser");
const table_1 = require("./table");
process.on('uncaughtException', (ex) => {
    console.error(ex);
});
process.on('unhandledRejection', (ex) => {
    console.error(ex);
});
let dbConnection = undefined;
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const argv = yargs.options({
            database: {
                alias: 'db',
                type: 'string',
                demandOption: true,
                description: 'database',
            },
            host: {
                alias: 'h',
                type: 'string',
            },
            user: {
                alias: 'u',
                type: 'string',
                demandOption: true,
            },
            password: {
                alias: 'p',
                type: 'string',
                demandOption: true,
            },
            analyse: {
                alias: 'a',
                type: 'boolean',
            },
            reset: {
                alias: 'r',
                type: 'boolean',
            },
        }).argv;
        let host = '127.0.0.1';
        let port = 3306;
        if (argv.host) {
            let portString;
            [host, portString] = argv.host.split(':');
            if (portString)
                port = parseInt(portString, 10);
        }
        dbConnection = knex_1.default({
            client: 'mysql',
            connection: {
                database: argv.database,
                host,
                port,
                user: argv.user,
                password: argv.password,
                supportBigNumbers: true,
            },
        }).on('query-error', (err) => {
            console.error(err.code, err.name);
        });
        if (argv.analyse) {
            let customSchema = undefined;
            try {
                customSchema = fs_extra_1.readJSONSync('./custom_schema.json');
            }
            catch (ex) {
                console.warn('Unable to read ./custom_schema.json, this will not take any customization into account.');
            }
            const analyser = new analyser_1.Analyser(dbConnection, argv.database, customSchema);
            yield analyser.extractTables();
            yield analyser.extractColumns();
            const json = analyser.generateJson();
            fs_extra_1.writeJSONSync('./schema.json', json, { spaces: 4 });
            return;
        }
        let schema = undefined;
        try {
            schema = fs_extra_1.readJSONSync('./schema.json');
        }
        catch (ex) {
            console.error('Unable to read from schema.json. Please run with --analyse first.');
        }
        if (!schema)
            return;
        const tableService = new table_1.TableService(dbConnection, schema.maxCharLength || 255, schema.values);
        for (const table of schema.tables) {
            if (table.lines > 0) {
                if (argv.reset)
                    yield tableService.empty(table);
                yield tableService.before(table);
                yield tableService.fill(table);
                yield tableService.after(table);
            }
        }
    });
}
main()
    .then(() => {
    if (dbConnection) {
        return dbConnection.destroy();
    }
});
