"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const randomizer_1 = require("./randomizer");
class TableService {
    constructor(dbConnection, maxCharLength, values) {
        this.dbConnection = dbConnection;
        this.maxCharLength = maxCharLength;
        this.values = values;
        this.foreignValues = {};
    }
    getLines(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const nbLines = (yield this.dbConnection(table.name).count())[0]['count(*)'];
            return nbLines;
        });
    }
    empty(table) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('empty: ', table.name);
            yield this.dbConnection.raw('SET FOREIGN_KEY_CHECKS = 0;');
            yield this.dbConnection.raw(`DELETE FROM ${table.name}`);
            yield this.dbConnection.raw(`ALTER TABLE ${table.name} AUTO_INCREMENT = 1;`);
        });
    }
    getForeignKeyValues(table, tableForeignKeyValues = {}, runRows) {
        return __awaiter(this, void 0, void 0, function* () {
            for (var c = 0; c < table.columns.length; c++) {
                const column = table.columns[c];
                if (column.foreignKey) {
                    const foreignKey = column.foreignKey;
                    let values = [];
                    if (column.options.unique) {
                        if (runRows) {
                            const query = this.dbConnection(foreignKey.table)
                                .distinct(`${foreignKey.table}.${foreignKey.column}`)
                                .leftJoin(table.name, function () {
                                this.on(`${table.name}.${column.name}`, `${foreignKey.table}.${foreignKey.column}`);
                            })
                                .whereNull(`${table.name}.${column.name}`)
                                .limit(runRows);
                            if (foreignKey.where) {
                                query.andWhere(this.dbConnection.raw(foreignKey.where));
                            }
                            values = (yield query).map(result => result[column.foreignKey.column]);
                            if (values.length === 0) {
                                throw new Error(`${table.name}: Not enough unique values available for foreign key ${foreignKey.table}.${foreignKey.column}`);
                            }
                        }
                    }
                    else {
                        const query = this.dbConnection(column.foreignKey.table).distinct(column.foreignKey.column);
                        if (foreignKey.where) {
                            query.andWhere(this.dbConnection.raw(foreignKey.where));
                        }
                        values = (yield query).map(result => result[column.foreignKey.column]);
                        if (values.length === 0 && !column.options.nullable) {
                            throw new Error(`${table.name}: Not enough values available for foreign key ${foreignKey.table}.${foreignKey.column}`);
                        }
                    }
                    tableForeignKeyValues[`${column.name}_${foreignKey.table}_${foreignKey.column}`] = values;
                }
            }
        });
    }
    before(table) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!table.before)
                return;
            for (const query of table.before) {
                yield this.dbConnection.raw(query);
            }
        });
    }
    fill(table) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('fill: ', table.name);
            const tableForeignKeyValues = {};
            try {
                yield this.getForeignKeyValues(table, tableForeignKeyValues);
            }
            catch (ex) {
                console.warn(ex.message);
                return;
            }
            let previousRunRows = -1;
            let currentNbRows = yield this.getLines(table);
            batch: while (currentNbRows < table.lines) {
                if (previousRunRows === currentNbRows) {
                    console.warn(`Can't insert more rows in ${table.name}`);
                    break batch;
                }
                previousRunRows = currentNbRows;
                const rows = [];
                const runRows = Math.min(1000, table.lines - currentNbRows);
                try {
                    yield this.getForeignKeyValues(table, tableForeignKeyValues, runRows);
                }
                catch (ex) {
                    console.warn(ex.message);
                    break batch;
                }
                for (let i = 0; i < runRows; i++) {
                    const row = {};
                    for (var c = 0; c < table.columns.length; c++) {
                        const column = table.columns[c];
                        if (column.options.autoIncrement)
                            continue;
                        if (column.foreignKey) {
                            const foreignKeys = tableForeignKeyValues[`${column.name}_${column.foreignKey.table}_${column.foreignKey.column}`];
                            row[column.name] = foreignKeys[randomizer_1.Randomizer.randomInt(0, foreignKeys.length - 1)];
                            continue;
                        }
                        if (column.values) {
                            if (Array.isArray(column.values)) {
                                row[column.name] = column.values[randomizer_1.Randomizer.randomInt(0, column.values.length - 1)];
                            }
                            else {
                                row[column.name] = this.values[column.values][randomizer_1.Randomizer.randomInt(0, this.values[column.values].length - 1)];
                            }
                            continue;
                        }
                        switch (column.generator) {
                            case 'bit':
                                row[column.name] = randomizer_1.Randomizer.randomBit(column.options.max);
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'tinyint':
                                if (column.options.unsigned) {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(0, 255);
                                }
                                else {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(-128, 127);
                                }
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'bool':
                            case 'boolean':
                                row[column.name] = randomizer_1.Randomizer.randomInt(0, 1);
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'smallint':
                                if (column.options.unsigned) {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(0, 65535);
                                }
                                else {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(-32768, 32767);
                                }
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'mediumint':
                                if (column.options.unsigned) {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(0, 16777215);
                                }
                                else {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(-8388608, 8388607);
                                }
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'int':
                            case 'integer':
                            case 'bigint':
                                if (column.options.unsigned) {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(0, 2147483647);
                                }
                                else {
                                    row[column.name] = randomizer_1.Randomizer.randomInt(-2147483648, 2147483647);
                                }
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'decimal':
                            case 'dec':
                            case 'float':
                            case 'double':
                                row[column.name] = Math.random();
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'date':
                            case 'datetime':
                            case 'timestamp':
                                const min = column.options.min ? new Date(column.options.min) : new Date('01-01-1970');
                                const max = column.options.max ? new Date(column.options.max) : new Date();
                                row[column.name] = randomizer_1.Randomizer.randomDate(min, max);
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'time':
                                const hours = randomizer_1.Randomizer.randomInt(-838, +838);
                                const minutes = randomizer_1.Randomizer.randomInt(-0, +59);
                                const seconds = randomizer_1.Randomizer.randomInt(-0, +59);
                                row[column.name] = `${hours}:${minutes}:${seconds}`;
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'year':
                                row[column.name] = randomizer_1.Randomizer.randomInt(1901, 2155);
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'varchar':
                            case 'char':
                            case 'binary':
                            case 'varbinary':
                                row[column.name] = randomizer_1.Randomizer.randomString(randomizer_1.Randomizer.randomInt(column.options.min, Math.min(this.maxCharLength, column.options.max)));
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'tinyblob':
                                row[column.name] = randomizer_1.Randomizer.randomString(randomizer_1.Randomizer.randomInt(0, 10));
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'text':
                            case 'mediumtext':
                            case 'longtext':
                                row[column.name] = randomizer_1.Randomizer.randomString(randomizer_1.Randomizer.randomInt(0, Math.min(this.maxCharLength, column.options.max)));
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'blob':
                            case 'mediumblob': // 16777215
                            case 'longblob': // 4,294,967,295
                                row[column.name] = randomizer_1.Randomizer.randomString(randomizer_1.Randomizer.randomInt(0, Math.min(this.maxCharLength, column.options.max)));
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'set':
                                row[column.name] = randomizer_1.Randomizer.randomBit(column.options.max);
                                if (column.options.nullable && Math.random() <= 0.1)
                                    row[column.name] = null;
                                break;
                            case 'enum':
                                if (column.options.nullable) {
                                    row[column.name] = Math.floor(Math.random() * (column.options.max + 1));
                                }
                                else {
                                    row[column.name] = Math.floor(Math.random() * (column.options.max)) + 1;
                                }
                                break;
                        }
                    }
                    rows.push(row);
                }
                const query = yield this.dbConnection(table.name)
                    .insert(rows)
                    .toQuery()
                    .replace('insert into', 'insert ignore into');
                const insertResult = yield this.dbConnection.raw(query);
                currentNbRows += insertResult[0].affectedRows;
                console.log(currentNbRows + ' / ' + table.lines);
            }
        });
    }
    after(table) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!table.after)
                return;
            for (const query of table.after) {
                yield this.dbConnection.raw(query);
            }
        });
    }
}
exports.TableService = TableService;
